import pygame
from OpenGL.GL import *
import math, os

class Texture:
    cache = {}
    @classmethod
    def get(cls, image_name, *params, **kwargs):
        if cls.cache.has_key(image_name):
            return cls.cache[image_name]
        else:
            tex = cls(image_name, *params, **kwargs)
            cls.cache[image_name] = tex
            return tex

    texture_id = None
    def __init__(self, image, scale_format="nearest", internal_format=GL_RGBA):
        if isinstance(image, str):
            image = os.path.join(*image.split("/"))
            self.filename = image
            image = pygame.image.load(image)
        else:
            self.filename = repr(image)
        w, h = image.get_width(), image.get_height()

        self.scale_format = scale_format

        # We need to make sure the texture size is always a power of 2.
        tw = int(2 ** math.ceil(math.log(w, 2)))
        th = int(2 ** math.ceil(math.log(h, 2)))

        if w != tw or h != th:
            w,h = tw,th
            s = pygame.Surface((w,h))
            s.blit(image, (0,h-image.get_height()))
        else:
            s = image
        self.width = w
        self.height = h

        data = pygame.image.tostring(s, 'RGBA', 1)
        self.texture_id = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.texture_id)
        glPixelStorei(GL_UNPACK_ROW_LENGTH, 0)
        glTexImage2D(GL_TEXTURE_2D, 0, internal_format, w, h,
             0, GL_RGBA, GL_UNSIGNED_BYTE, data)
        if scale_format == "nearest":
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        elif scale_format == "linear":
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)

    def __repr__(self):
        return '<Texture %rx%r %r>'%(self.width, self.height,
            self.filename)

    def __del__(self):
        try:
            if self.texture_id is None: return
            try:
                glDeleteTextures(self.texture_id)
                self.texture_id = None
            except NameError:
                # glDeleteTextures has been gobbled
                pass
        except TypeError: pass

    def render(self, x=0, y=0, mode=GL_MODULATE):
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.texture_id)
        #glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, mode)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 0)
        glVertex2f(x+0, y+0)
        glTexCoord2f(1, 0)
        glVertex2f(x+self.width, y+0)
        glTexCoord2f(1, 1)
        glVertex2f(x+self.width, y+self.height)
        glTexCoord2f(0, 1)
        glVertex2f(x+0, y+self.height)
        glEnd()
        glDisable(GL_TEXTURE_2D)

    def sub(self, x, y, w, h):
        return SubTexture(self, x, y, w, h)


class SubTexture(Texture):
    def __init__(self, texture, x, y, w, h):
        self.__texture = texture            # don't collect the parent!
        self.filename = texture.filename
        self.x, self.y = x, y
        self.w, self.h = w, h
        self.texture_id = texture.texture_id

    def __repr__(self):
        return '<Texture %rx%r %r>'%(self.w, self.h,
            self.filename)

    def render(self, x=0, y=0):
        glEnable(GL_TEXTURE_2D)
        #glEnable(GL_BLEND)
        glBindTexture(GL_TEXTURE_2D, self.__texture.texture_id)
        #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
        #glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
        glBegin(GL_QUADS)
        w, h = float(self.__texture.width), float(self.__texture.height)
        su, sv = self.x/w, self.y/h
        eu, ev = su + self.w/w, sv + self.h/h
        glTexCoord2f(su, sv)
        glVertex2f(x, y)
        glTexCoord2f(eu, sv)
        glVertex2f(x+self.w, y)
        glTexCoord2f(eu, ev)
        glVertex2f(x+self.w, y+self.h)
        glTexCoord2f(su, ev)
        glVertex2f(x, y+self.h)
        glEnd()
        #glDisable(GL_BLEND)
        glDisable(GL_TEXTURE_2D)

    def __del__(self):
        pass